#include <windows.h>
#include <stdio.h>

#include "resource.h"

#include "VideoFilter.h"
#include "VideoFilterDialog.h"

#include "JPSDR_Filter.h"

extern int g_VFVAPIVersion;


extern "C" void JPSDR_AutoYUY2_Convert420_to_YUY2_1(const void *scr_y,const void *src_u,const void *src_v,
		void *dst,sint32 w);
extern "C" void JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_2(const void *scr_y,const void *src_u1,const void *src_u2,
		const void *src_v1,const void *src_v2,void *dst,sint32 w);
extern "C" void JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_3(const void *scr_y,const void *src_u1,const void *src_u2,
		const void *src_v1,const void *src_v2,void *dst,sint32 w);
extern "C" void JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_4(const void *scr_y,const void *src_u1,const void *src_u2,
		const void *src_v1,const void *src_v2,void *dst,sint32 w);

extern "C" void JPSDR_AutoYUY2_Convert420_to_UYVY_1(const void *scr_y,const void *src_u,const void *src_v,
		void *dst,sint32 w);
extern "C" void JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_2(const void *scr_y,const void *src_u1,const void *src_u2,
		const void *src_v1,const void *src_v2,void *dst,sint32 w);
extern "C" void JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_3(const void *scr_y,const void *src_u1,const void *src_u2,
		const void *src_v1,const void *src_v2,void *dst,sint32 w);
extern "C" void JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_4(const void *scr_y,const void *src_u1,const void *src_u2,
		const void *src_v1,const void *src_v2,void *dst,sint32 w);

extern "C" void JPSDR_AutoYUY2_Move8_Full(const void *src, void *dst, sint32 w,sint32 h,
		ptrdiff_t src_modulo,ptrdiff_t dst_modulo);
extern "C" void JPSDR_AutoYUY2_Move32_Full(const void *src, void *dst, sint32 w,sint32 h,
		ptrdiff_t src_modulo,ptrdiff_t dst_modulo);


extern "C" void JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_2(const void *scr_1,const void *src_2,void *dst,sint32 w);
extern "C" void JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_3(const void *scr_1,const void *src_2,void *dst,sint32 w);
extern "C" void JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_4(const void *scr_1,const void *src_2,void *dst,sint32 w);



#define Interlaced_Tab_Size 3

class JPSDR_AutoYUY2Data
{
public:
	Image_Data image_data;
	unsigned char output_format;
	unsigned char convert_mode;
	unsigned char threshold;
	bool *interlaced_tab[Interlaced_Tab_Size];

	JPSDR_AutoYUY2Data(void);
	virtual ~JPSDR_AutoYUY2Data(void);
};


JPSDR_AutoYUY2Data::JPSDR_AutoYUY2Data(void)
{
	signed char i;

	output_format=0;
	convert_mode=0;
	threshold=4;

	for (i=0; i<Interlaced_Tab_Size; i++)
	{
		interlaced_tab[i]=NULL;
	}
}


JPSDR_AutoYUY2Data::~JPSDR_AutoYUY2Data(void)
{
	signed char i;

	for (i=Interlaced_Tab_Size-1; i>=0; i--)
	{
		myfree(interlaced_tab[i]);
	}
}


class JPSDR_AutoYUY2Dialog : public VDXVideoFilterDialog
{
public:
	JPSDR_AutoYUY2Dialog(JPSDR_AutoYUY2Data& Data, IVDXFilterPreview *ifp) : mData(Data), mifp(ifp)
	{}
	
	bool Show(HWND parent)
	{
		return 0!=VDXVideoFilterDialog::Show(NULL,MAKEINTRESOURCE(IDD_JPSDR_AUTO_YUY2),parent);
	}
	
	virtual INT_PTR DlgProc(UINT msg, WPARAM wParam, LPARAM lParam);
	
protected:
	bool OnInit();
	bool OnCommand(int cmd);
	void OnDestroy();
	
	bool SaveToData();
	
	JPSDR_AutoYUY2Data& mData;
	JPSDR_AutoYUY2Data moldData;
	IVDXFilterPreview *const mifp;
};


INT_PTR JPSDR_AutoYUY2Dialog::DlgProc(UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		case WM_INITDIALOG : return !OnInit();
		case WM_DESTROY :
			OnDestroy();
			break;
		case WM_COMMAND :
			if (OnCommand(LOWORD(wParam))) return TRUE;
			break;
	}
	return FALSE;
}



bool JPSDR_AutoYUY2Dialog::OnInit()
{
	bool enable_dlg;
	moldData=mData;

	switch (mData.output_format)
	{
		case 0 : CheckDlgButton(mhdlg,IDC_YUY2,1); break;
		case 1 : CheckDlgButton(mhdlg,IDC_UYVY,1); break;
		case 2 : CheckDlgButton(mhdlg,IDC_PLANAR_422,1); break;
	}
	switch (mData.convert_mode)
	{
		case 0 : CheckDlgButton(mhdlg,IDC_PROGRESSIVE,1); enable_dlg=false; break;
		case 1 : CheckDlgButton(mhdlg,IDC_INTERLACED,1); enable_dlg=false; break;
		case 2 : CheckDlgButton(mhdlg,IDC_AUTOMATIC,1); enable_dlg=true; break;
		case 3 : CheckDlgButton(mhdlg,IDC_TEST,1); enable_dlg=true; break;
	}

	SetDlgItemInt(mhdlg,IDC_THRESHOLD,mData.threshold,FALSE);

	EnableWindow(GetDlgItem(mhdlg,IDC_THRESHOLD),enable_dlg);
	EnableWindow(GetDlgItem(mhdlg,IDC_THRESHOLD_TEXT),enable_dlg);

	return false;
}


void JPSDR_AutoYUY2Dialog::OnDestroy()
{
	if (mifp) mifp->InitButton(NULL);
}

bool JPSDR_AutoYUY2Dialog::SaveToData()
{
	unsigned char thrs;
	BOOL success;

	thrs=GetDlgItemInt(mhdlg,IDC_THRESHOLD,&success,FALSE);
	if (success) mData.threshold=thrs;

	if (!!IsDlgButtonChecked(mhdlg,IDC_YUY2)) mData.output_format=0;
	if (!!IsDlgButtonChecked(mhdlg,IDC_UYVY)) mData.output_format=1;
	if (!!IsDlgButtonChecked(mhdlg,IDC_PLANAR_422)) mData.output_format=2;
	if (!!IsDlgButtonChecked(mhdlg,IDC_PROGRESSIVE)) mData.convert_mode=0;
	if (!!IsDlgButtonChecked(mhdlg,IDC_INTERLACED)) mData.convert_mode=1;
	if (!!IsDlgButtonChecked(mhdlg,IDC_AUTOMATIC)) mData.convert_mode=2;
	if (!!IsDlgButtonChecked(mhdlg,IDC_TEST)) mData.convert_mode=3;

	return true;
}



bool JPSDR_AutoYUY2Dialog::OnCommand(int cmd)
{
	switch(cmd)
	{
		case IDOK :
			SaveToData();
			EndDialog(mhdlg,true);
			return true;
		case IDCANCEL :
			mData=moldData;
			EndDialog(mhdlg,false);
			return true;
		case IDC_PROGRESSIVE :
		case IDC_INTERLACED :
			EnableWindow(GetDlgItem(mhdlg,IDC_THRESHOLD),false);
			EnableWindow(GetDlgItem(mhdlg,IDC_THRESHOLD_TEXT),false);
			return true;
		case IDC_AUTOMATIC :
		case IDC_TEST :
			EnableWindow(GetDlgItem(mhdlg,IDC_THRESHOLD),true);
			EnableWindow(GetDlgItem(mhdlg,IDC_THRESHOLD_TEXT),true);
			return true;
	}
		
	return false;
}



class JPSDR_AutoYUY2 : public VDXVideoFilter
{
public:
	virtual bool Init();
	virtual uint32 GetParams();
	virtual void Start();
	virtual void Run();
	virtual void End();
	virtual bool Configure(VDXHWND hwnd);
	virtual void GetSettingString(char *buf, int maxlen);
	virtual void GetScriptString(char *buf, int maxlen);
	
	VDXVF_DECLARE_SCRIPT_METHODS();
	
protected:
	unsigned short lookup[768];
	bool SSE2_Enable;

	void Convert_Progressive_to_Planar422(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V);
	void Convert_Progressive_to_Planar422_SSE(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V);
	void Convert_Progressive_to_Planar422_NonSSE(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V);

	void Convert_Interlaced_to_Planar422(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V);
	void Convert_Interlaced_to_Planar422_SSE(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V);
	void Convert_Interlaced_to_Planar422_NonSSE(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V);

	void Convert_Automatic_to_Planar422(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_UV,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V,unsigned char thrs);
	void Convert_Test_to_Planar422(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_UV,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V,unsigned char thrs);


	void JPSDR_AutoYUY2::Convert_Progressive_to_YUY2(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);
	void JPSDR_AutoYUY2::Convert_Progressive_to_YUY2_SSE(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);
	void JPSDR_AutoYUY2::Convert_Progressive_to_YUY2_NonSSE(const void *_src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);

	void JPSDR_AutoYUY2::Convert_Interlaced_to_YUY2(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);
	void JPSDR_AutoYUY2::Convert_Interlaced_to_YUY2_SSE(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);
	void JPSDR_AutoYUY2::Convert_Interlaced_to_YUY2_NonSSE(const void *_src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);

	void JPSDR_AutoYUY2::Convert_Automatic_to_YUY2(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch,
		unsigned char thrs);
	void JPSDR_AutoYUY2::Convert_Test_to_YUY2(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch,
		unsigned char thrs);

	void JPSDR_AutoYUY2::Convert_Progressive_to_UYVY(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);
	void JPSDR_AutoYUY2::Convert_Progressive_to_UYVY_SSE(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);
	void JPSDR_AutoYUY2::Convert_Progressive_to_UYVY_NonSSE(const void *_src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);

	void JPSDR_AutoYUY2::Convert_Interlaced_to_UYVY(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);
	void JPSDR_AutoYUY2::Convert_Interlaced_to_UYVY_SSE(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);
	void JPSDR_AutoYUY2::Convert_Interlaced_to_UYVY_NonSSE(const void *_src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch);

	void JPSDR_AutoYUY2::Convert_Automatic_to_UYVY(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch,
		unsigned char thrs);
	void JPSDR_AutoYUY2::Convert_Test_to_UYVY(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch,
		unsigned char thrs);

	void ScriptConfig(IVDXScriptInterpreter *isi, const VDXScriptValue *argv, int argc);
		
	JPSDR_AutoYUY2Data mData;
};

VDXVF_BEGIN_SCRIPT_METHODS(JPSDR_AutoYUY2)
VDXVF_DEFINE_SCRIPT_METHOD(JPSDR_AutoYUY2,ScriptConfig,"iii")
VDXVF_END_SCRIPT_METHODS()


bool JPSDR_AutoYUY2::Init()
{
	signed short i;

/*	if (g_VFVAPIVersion<12)
	{
		ff->Except("This virtualdub version doesn't support this filter !");
		return(false);
	}*/

	for (i=0; i<256; i++)
	{
		lookup[i]=3*i;
		lookup[i+256]=5*i;
		lookup[i+512]=7*i;
	}

	return(true);
}



uint32 JPSDR_AutoYUY2::GetParams()
{
	if (g_VFVAPIVersion<12) return(FILTERPARAM_NOT_SUPPORTED);

	switch(fa->src.mpPixmapLayout->format)
	{
/*		case nsVDXPixmap::kPixFormat_XRGB8888 :
		case nsVDXPixmap::kPixFormat_YUV422_YUYV :
		case nsVDXPixmap::kPixFormat_YUV422_UYVY :
		case nsVDXPixmap::kPixFormat_YUV444_Planar :
		case nsVDXPixmap::kPixFormat_YUV422_Planar :*/
		case nsVDXPixmap::kPixFormat_YUV420_Planar :
/*		case nsVDXPixmap::kPixFormat_YUV411_Planar :
		case nsVDXPixmap::kPixFormat_YUV410_Planar :
		case nsVDXPixmap::kPixFormat_Y8 :*/
			break;
		default : return FILTERPARAM_NOT_SUPPORTED;
	}

	fa->dst.offset=fa->src.offset;
	fa->dst.depth=0;
	switch(mData.output_format)
	{
		case 0 : fa->dst.mpPixmapLayout->format=nsVDXPixmap::kPixFormat_YUV422_YUYV; break;
		case 1 : fa->dst.mpPixmapLayout->format=nsVDXPixmap::kPixFormat_YUV422_UYVY; break;
		case 2 : fa->dst.mpPixmapLayout->format=nsVDXPixmap::kPixFormat_YUV422_Planar; break;
	}
	fa->dst.mpPixmapLayout->pitch=0;

	return(FILTERPARAM_SUPPORTS_ALTFORMATS|FILTERPARAM_SWAP_BUFFERS);
}


void JPSDR_AutoYUY2::Start()
{
	Image_Data idata;
	signed char i;
	bool ok;

	if (g_VFVAPIVersion>=12)
	{
		const VDXPixmap& pxdst=*fa->dst.mpPixmap;
		const VDXPixmap& pxsrc=*fa->src.mpPixmap;

		idata.src_h0=pxsrc.h;
		idata.src_w0=pxsrc.w;
		idata.src_pitch0=pxsrc.pitch;
		idata.src_pitch1=pxsrc.pitch2;
		idata.src_pitch2=pxsrc.pitch3;

		idata.dst_h0=pxdst.h;
		idata.dst_w0=pxdst.w;
		idata.dst_pitch0=pxdst.pitch;
		idata.dst_pitch1=pxdst.pitch2;
		idata.dst_pitch2=pxdst.pitch3;
		
		switch(pxsrc.format)
		{
/*			case nsVDXPixmap::kPixFormat_XRGB8888 :
				idata.src_video_mode=1;
				idata.video_mode=1;
				idata.src_modulo0=idata.src_pitch0-4*idata.src_w0;
				idata.src_size0=idata.src_h0*4*idata.src_w0;
				idata.src_h1=0;
				idata.src_h2=0;
				idata.src_w1=0;
				idata.src_w2=0;
				idata.src_pitch1=0;
				idata.src_pitch2=0;
				idata.src_modulo1=0;
				idata.src_modulo2=0;
				idata.src_size1=0;
				idata.src_size2=0;
				break;
			case nsVDXPixmap::kPixFormat_YUV422_YUYV :
				idata.src_video_mode=2;
				idata.video_mode=2;
				idata.src_modulo0=idata.src_pitch0-4*((idata.src_w0+1)>>1);
				idata.src_size0=idata.src_h0*4*((idata.src_w0+1)>>1);
				idata.src_h1=0;
				idata.src_h2=0;
				idata.src_w1=0;
				idata.src_w2=0;
				idata.src_pitch1=0;
				idata.src_pitch2=0;
				idata.src_modulo1=0;
				idata.src_modulo2=0;
				idata.src_size1=0;
				idata.src_size2=0;
				break;
			case nsVDXPixmap::kPixFormat_YUV422_UYVY :
				idata.src_video_mode=3;
				idata.video_mode=3;
				idata.src_modulo0=idata.src_pitch0-4*((idata.src_w0+1)>>1);
				idata.src_size0=idata.src_h0*4*((idata.src_w0+1)>>1);
				idata.src_h1=0;
				idata.src_h2=0;
				idata.src_w1=0;
				idata.src_w2=0;
				idata.src_pitch1=0;
				idata.src_pitch2=0;
				idata.src_modulo1=0;
				idata.src_modulo2=0;
				idata.src_size1=0;
				idata.src_size2=0;
				break;
			case nsVDXPixmap::kPixFormat_YUV444_Planar :
				idata.scr_video_mode=4;
				idata.video_mode=4;
				idata.src_h1=idata.src_h0;
				idata.src_h2=idata.src_h0;
				idata.src_w1=idata.src_w0;
				idata.src_w2=idata.src_w0;
				idata.src_modulo0=idata.src_pitch0-idata.src_w0;
				idata.src_modulo1=idata.src_pitch1-idata.src_w1;
				idata.src_modulo2=idata.src_pitch2-idata.src_w2;
				idata.src_size0=idata.src_w0*idata.src_h0;
				idata.src_size1=idata.src_w1*idata.src_h1;
				idata.src_size2=idata.src_w2*idata.src_h2;
				break;
			case nsVDXPixmap::kPixFormat_YUV422_Planar :
				idata.src_video_mode=5;
				idata.video_mode=5;
				idata.src_h1=idata.src_h0;
				idata.src_h2=idata.src_h0;
				idata.src_w1=(idata.src_w0+1)>>1;
				idata.src_w2=(idata.src_w0+1)>>1;
				idata.src_modulo0=idata.src_pitch0-idata.src_w0;
				idata.src_modulo1=idata.src_pitch1-idata.src_w1;
				idata.src_modulo2=idata.src_pitch2-idata.src_w2;
				idata.src_size0=idata.src_w0*idata.src_h0;
				idata.src_size1=idata.src_w1*idata.src_h1;
				idata.src_size2=idata.src_w2*idata.src_h2;
				break;*/
			case nsVDXPixmap::kPixFormat_YUV420_Planar :
				idata.src_video_mode=6;
				idata.video_mode=6;
				idata.src_h1=(idata.src_h0+1)>>1;
				idata.src_h2=(idata.src_h0+1)>>1;
				idata.src_w1=(idata.src_w0+1)>>1;
				idata.src_w2=(idata.src_w0+1)>>1;
				idata.src_modulo0=idata.src_pitch0-idata.src_w0;
				idata.src_modulo1=idata.src_pitch1-idata.src_w1;
				idata.src_modulo2=idata.src_pitch2-idata.src_w2;
				idata.src_size0=idata.src_w0*idata.src_h0;
				idata.src_size1=idata.src_w1*idata.src_h1;
				idata.src_size2=idata.src_w2*idata.src_h2;
				break;
/*			case nsVDXPixmap::kPixFormat_YUV411_Planar :
				idata.src_video_mode=7;
				idata.video_mode=7;
				idata.src_h1=idata.src_h0;
				idata.src_h2=idata.src_h0;
				idata.src_w1=(idata.src_w0+3)>>2;
				idata.src_w2=(idata.src_w0+3)>>2;
				idata.src_modulo0=idata.src_pitch0-idata.src_w0;
				idata.src_modulo1=idata.src_pitch1-idata.src_w1;
				idata.src_modulo2=idata.src_pitch2-idata.src_w2;
				idata.src_size0=idata.src_w0*idata.src_h0;
				idata.src_size1=idata.src_w1*idata.src_h1;
				idata.src_size2=idata.src_w2*idata.src_h2;
				break;
			case nsVDXPixmap::kPixFormat_YUV410_Planar :
				idata.src_video_mode=8;
				idata.video_mode=8;
				idata.src_h1=(idata.src_h0+3)>>2;
				idata.src_h2=(idata.src_h0+3)>>2;
				idata.src_w1=(idata.src_w0+3)>>2;
				idata.src_w2=(idata.src_w0+3)>>2;
				idata.src_modulo0=idata.src_pitch0-idata.src_w0;
				idata.src_modulo1=idata.src_pitch1-idata.src_w1;
				idata.src_modulo2=idata.src_pitch2-idata.src_w2;
				idata.src_size0=idata.src_w0*idata.src_h0;
				idata.src_size1=idata.src_w1*idata.src_h1;
				idata.src_size2=idata.src_w2*idata.src_h2;
				break;
			case nsVDXPixmap::kPixFormat_Y8 :
				idata.src_video_mode=9;
				idata.video_mode=9;
				idata.src_modulo0=idata.src_pitch0-idata.src_w0;
				idata.src_size0=idata.src_h0*idata.src_w0;
				idata.src_h1=0;
				idata.src_h2=0;
				idata.src_w1=0;
				idata.src_w2=0;
				idata.src_pitch1=0;
				idata.src_pitch2=0;
				idata.src_modulo1=0;
				idata.src_modulo2=0;
				idata.src_size1=0;
				idata.src_size2=0;
				break;*/
		}
		switch(pxdst.format)
		{
/*			case nsVDXPixmap::kPixFormat_XRGB8888 :
				idata.dst_video_mode=1;
				idata.dst_modulo0=idata.dst_pitch0-4*idata.dst_w0;
				idata.dst_size0=idata.dst_h0*4*idata.dst_w0;
				idata.dst_h1=0;
				idata.dst_h2=0;
				idata.dst_w1=0;
				idata.dst_w2=0;
				idata.dst_pitch1=0;
				idata.dst_pitch2=0;
				idata.dst_modulo1=0;
				idata.dst_modulo2=0;
				idata.dst_size1=0;
				idata.dst_size2=0;
				break;*/
			case nsVDXPixmap::kPixFormat_YUV422_YUYV :
				idata.dst_video_mode=2;
				idata.dst_modulo0=idata.dst_pitch0-4*((idata.dst_w0+1)>>1);
				idata.dst_size0=idata.dst_h0*4*((idata.dst_w0+1)>>1);
				idata.dst_h1=0;
				idata.dst_h2=0;
				idata.dst_w1=0;
				idata.dst_w2=0;
				idata.dst_pitch1=0;
				idata.dst_pitch2=0;
				idata.dst_modulo1=0;
				idata.dst_modulo2=0;
				idata.dst_size1=0;
				idata.dst_size2=0;
				break;
			case nsVDXPixmap::kPixFormat_YUV422_UYVY :
				idata.dst_video_mode=3;
				idata.dst_modulo0=idata.dst_pitch0-4*((idata.dst_w0+1)>>1);
				idata.dst_size0=idata.dst_h0*4*((idata.dst_w0+1)>>1);
				idata.dst_h1=0;
				idata.dst_h2=0;
				idata.dst_w1=0;
				idata.dst_w2=0;
				idata.dst_pitch1=0;
				idata.dst_pitch2=0;
				idata.dst_modulo1=0;
				idata.dst_modulo2=0;
				idata.dst_size1=0;
				idata.dst_size2=0;
				break;
/*			case nsVDXPixmap::kPixFormat_YUV444_Planar :
				idata.dst_video_mode=4;
				idata.dst_h1=idata.dst_h0;
				idata.dst_h2=idata.dst_h0;
				idata.dst_w1=idata.dst_w0;
				idata.dst_w2=idata.dst_w0;
				idata.dst_modulo0=idata.dst_pitch0-idata.dst_w0;
				idata.dst_modulo1=idata.dst_pitch1-idata.dst_w1;
				idata.dst_modulo2=idata.dst_pitch2-idata.dst_w2;
				idata.dst_size0=idata.dst_w0*idata.dst_h0;
				idata.dst_size1=idata.dst_w1*idata.dst_h1;
				idata.dst_size2=idata.dst_w2*idata.dst_h2;
				break;*/
			case nsVDXPixmap::kPixFormat_YUV422_Planar :
				idata.dst_video_mode=5;
				idata.dst_h1=idata.dst_h0;
				idata.dst_h2=idata.dst_h0;
				idata.dst_w1=(idata.dst_w0+1)>>1;
				idata.dst_w2=(idata.dst_w0+1)>>1;
				idata.dst_modulo0=idata.dst_pitch0-idata.dst_w0;
				idata.dst_modulo1=idata.dst_pitch1-idata.dst_w1;
				idata.dst_modulo2=idata.dst_pitch2-idata.dst_w2;
				idata.dst_size0=idata.dst_w0*idata.dst_h0;
				idata.dst_size1=idata.dst_w1*idata.dst_h1;
				idata.dst_size2=idata.dst_w2*idata.dst_h2;
				break;
/*			case nsVDXPixmap::kPixFormat_YUV420_Planar :
				idata.dst_video_mode=6;
				idata.dst_h1=(idata.dst_h0+1)>>1;
				idata.dst_h2=(idata.dst_h0+1)>>1;
				idata.dst_w1=(idata.dst_w0+1)>>1;
				idata.dst_w2=(idata.dst_w0+1)>>1;
				idata.dst_modulo0=idata.dst_pitch0-idata.dst_w0;
				idata.dst_modulo1=idata.dst_pitch1-idata.dst_w1;
				idata.dst_modulo2=idata.dst_pitch2-idata.dst_w2;
				idata.dst_size0=idata.dst_w0*idata.dst_h0;
				idata.dst_size1=idata.dst_w1*idata.dst_h1;
				idata.dst_size2=idata.dst_w2*idata.dst_h2;
				break;
			case nsVDXPixmap::kPixFormat_YUV411_Planar :
				idata.dst_video_mode=7;
				idata.dst_h1=idata.dst_h0;
				idata.dst_h2=idata.dst_h0;
				idata.dst_w1=(idata.dst_w0+3)>>2;
				idata.dst_w2=(idata.dst_w0+3)>>2;
				idata.dst_modulo0=idata.dst_pitch0-idata.dst_w0;
				idata.dst_modulo1=idata.dst_pitch1-idata.dst_w1;
				idata.dst_modulo2=idata.dst_pitch2-idata.dst_w2;
				idata.dst_size0=idata.dst_w0*idata.dst_h0;
				idata.dst_size1=idata.dst_w1*idata.dst_h1;
				idata.dst_size2=idata.dst_w2*idata.dst_h2;
				break;
			case nsVDXPixmap::kPixFormat_YUV410_Planar :
				idata.dst_video_mode=8;
				idata.dst_h1=(idata.dst_h0+3)>>2;
				idata.dst_h2=(idata.dst_h0+3)>>2;
				idata.dst_w1=(idata.dst_w0+3)>>2;
				idata.dst_w2=(idata.dst_w0+3)>>2;
				idata.dst_modulo0=idata.dst_pitch0-idata.dst_w0;
				idata.dst_modulo1=idata.dst_pitch1-idata.dst_w1;
				idata.dst_modulo2=idata.dst_pitch2-idata.dst_w2;
				idata.dst_size0=idata.dst_w0*idata.dst_h0;
				idata.dst_size1=idata.dst_w1*idata.dst_h1;
				idata.dst_size2=idata.dst_w2*idata.dst_h2;
				break;
			case nsVDXPixmap::kPixFormat_Y8 :
				idata.dstvideo_mode=9;
				idata.dst_modulo0=idata.dst_pitch0-idata.dst_w0;
				idata.dst_size0=idata.dst_h0*idata.dst_w0;
				idata.dst_h1=0;
				idata.dst_h2=0;
				idata.dst_w1=0;
				idata.dst_w2=0;
				idata.dst_pitch1=0;
				idata.dst_pitch2=0;
				idata.dst_modulo1=0;
				idata.dst_modulo2=0;
				idata.dst_size1=0;
				idata.dst_size2=0;
				break;*/
		}
	}
	else
	{
		idata.video_mode=0;
		idata.src_video_mode=0;
		idata.dst_video_mode=0;
		idata.src_w0=fa->src.w;
		idata.src_h0=fa->src.h;
		idata.src_pitch0=fa->src.pitch;
		idata.src_modulo0=fa->src.modulo;
		idata.src_size0=idata.src_h0*4*idata.src_w0;
		idata.dst_w0=fa->dst.w;
		idata.dst_h0=fa->dst.h;
		idata.dst_pitch0=fa->dst.pitch;
		idata.dst_modulo0=fa->dst.modulo;
		idata.dst_size0=idata.dst_h0*4*idata.dst_w0;
		idata.src_h1=0;
		idata.src_h2=0;
		idata.src_w1=0;
		idata.src_w2=0;
		idata.src_pitch1=0;
		idata.src_pitch2=0;
		idata.src_modulo1=0;
		idata.src_modulo2=0;
		idata.src_size1=0;
		idata.src_size2=0;
		idata.dst_h1=0;
		idata.dst_h2=0;
		idata.dst_w1=0;
		idata.dst_w2=0;
		idata.dst_pitch1=0;
		idata.dst_pitch2=0;
		idata.dst_modulo1=0;
		idata.dst_modulo2=0;
		idata.dst_size1=0;
		idata.dst_size2=0;
	}

	mData.image_data=idata;

	if (g_VFVAPIVersion<12)
	{
		ff->Except("This virtualdub version doesn't support this filter !");
		return;
	}

	if ((idata.src_h0<8)||(idata.src_w0<4))
	{
		ff->Except("Size must be at least 4x8 !");
		return;
	}

	for (i=0; i<Interlaced_Tab_Size; i++)
	{
		mData.interlaced_tab[i]=(bool*)malloc(idata.dst_w0*sizeof(bool));
	}

	ok=true;
	for (i=0; i<Interlaced_Tab_Size; i++)
	{
		ok=ok && (mData.interlaced_tab[i]!=NULL);
	}

	if (!ok)
	{
		for (i=Interlaced_Tab_Size-1; i>=0; i--)
		{
			myfree(mData.interlaced_tab[i]);
		}
		ff->ExceptOutOfMemory();
		return;
	}

	SSE2_Enable=((ff->getCPUFlags() & CPUF_SUPPORTS_SSE2)!=0);

}





void JPSDR_AutoYUY2::GetSettingString(char *buf, int maxlen)
{
	char output_string[255];

	switch(mData.output_format)
	{
		case 0 : strcpy_s(output_string,255," Output : YUY2"); break;
		case 1 : strcpy_s(output_string,255," Output : UYVY"); break;
		case 2 : strcpy_s(output_string,255," Output : Planar 4:2:2"); break;
	}
	switch(mData.convert_mode)
	{
		case 0 : strcat_s(output_string,255," Mode : Progressive"); break;
		case 1 : strcat_s(output_string,255," Mode : Interlaced"); break;
		case 2 : strcat_s(output_string,255," Mode : Automatic Threshold : %d"); break;
		case 3 : strcat_s(output_string,255," Mode : Test Threshold : %d"); break;
	}

	switch(mData.convert_mode)
	{
		case 0 :
		case 1 :
			SafePrintf(buf,maxlen,output_string);
			break;
		case 2 :
		case 3 :
			SafePrintf(buf,maxlen,output_string,mData.threshold);
			break;
	}

}



void JPSDR_AutoYUY2::Convert_Interlaced_to_Planar422(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V)
{
	if (SSE2_Enable)
		Convert_Interlaced_to_Planar422_SSE(src_Y,src_U,src_V,dst_Y,dst_U,dst_V,w_Y,w_U,w_V,h_Y,src_modulo_Y,src_pitch_U,
			src_pitch_V,dst_modulo_Y,dst_pitch_U,dst_pitch_V);
	else
		Convert_Interlaced_to_Planar422_NonSSE(src_Y,src_U,src_V,dst_Y,dst_U,dst_V,w_Y,w_U,w_V,h_Y,src_modulo_Y,src_pitch_U,
			src_pitch_V,dst_modulo_Y,dst_pitch_U,dst_pitch_V);
}


void JPSDR_AutoYUY2::Convert_Progressive_to_Planar422(const void *src_Y,const void *src_U,const void *src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V)
{
	if (SSE2_Enable)
		Convert_Progressive_to_Planar422_SSE(src_Y,src_U,src_V,dst_Y,dst_U,dst_V,w_Y,w_U,w_V,h_Y,src_modulo_Y,src_pitch_U,
			src_pitch_V,dst_modulo_Y,dst_pitch_U,dst_pitch_V);
	else
		Convert_Progressive_to_Planar422_NonSSE(src_Y,src_U,src_V,dst_Y,dst_U,dst_V,w_Y,w_U,w_V,h_Y,src_modulo_Y,src_pitch_U,
			src_pitch_V,dst_modulo_Y,dst_pitch_U,dst_pitch_V);
}


void JPSDR_AutoYUY2::Convert_Interlaced_to_Planar422_SSE(const void *src_Y,const void *_src_U,const void *_src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V)
{
	sint32 i,h_4,w_U8,w_V8;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	ptrdiff_t pitch_U_2,pitch_V_2;


	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	h_4=h_Y-4;
	w_U8=w_U>>3;
	w_V8=w_V>>3;

	JPSDR_AutoYUY2_Move32_Full(src_Y,dst_Y,w_Y>>2,h_Y,src_modulo_Y,dst_modulo_Y);

	for(i=0; i<4; i+=4)
	{
		memcpy(dst_U,src_U,w_U);
		memcpy(dst_V,src_V,w_V);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		memcpy(dst_U,src_Un,w_U);
		memcpy(dst_V,src_Vn,w_V);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_2(src_Unn,src_U,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_2(src_Vnn,src_V,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_3(src_Un,src_Unnn,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_3(src_Vn,src_Vnnn,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=4; i<h_4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_3(src_U,src_Upp,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_3(src_V,src_Vpp,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_2(src_Up,src_Un,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_2(src_Vp,src_Vn,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_2(src_Unn,src_U,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_2(src_Vnn,src_V,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_3(src_Un,src_Unnn,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_3(src_Vn,src_Vnnn,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_3(src_U,src_Upp,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_3(src_V,src_Vpp,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_2(src_Up,src_Un,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_2(src_Vp,src_Vn,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		memcpy(dst_U,src_U,w_U);
		memcpy(dst_V,src_V,w_V);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		memcpy(dst_U,src_Un,w_U);
		memcpy(dst_V,src_Vn,w_V);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}


void JPSDR_AutoYUY2::Convert_Interlaced_to_Planar422_NonSSE(const void *src_Y,const void *_src_U,const void *_src_V,void *dst_Y,void *_dst_U,
		void *_dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V)
{
	sint32 i,h_4,j;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	unsigned char *dst_V,*dst_U;
	ptrdiff_t pitch_U_2,pitch_V_2;


	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	dst_U=(unsigned char *)_dst_U;
	dst_V=(unsigned char *)_dst_V;
	h_4=h_Y-4;

	if ((w_Y&03)==0) JPSDR_AutoYUY2_Move32_Full(src_Y,dst_Y,w_Y>>2,h_Y,src_modulo_Y,dst_modulo_Y);
	else JPSDR_AutoYUY2_Move8_Full(src_Y,dst_Y,w_Y,h_Y,src_modulo_Y,dst_modulo_Y);

	for(i=0; i<4; i+=4)
	{
		memcpy(dst_U,src_U,w_U);
		memcpy(dst_V,src_V,w_V);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_Un,w_U);
		memcpy(dst_V,src_Vn,w_V);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Unn[j]]+lookup[src_U[j]+256]+4)>>3);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_Vnn[j]]+lookup[src_V[j]+256]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Un[j]+512]+(unsigned short)src_Unnn[j]+4)>>3);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_Vn[j]+512]+(unsigned short)src_Vnnn[j]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=4; i<h_4; i+=4)
	{
		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_U[j]+512]+(unsigned short)src_Upp[j]+4)>>3);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_V[j]+512]+(unsigned short)src_Vpp[j]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Up[j]]+lookup[src_Un[j]+256]+4)>>3);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_Vp[j]]+lookup[src_Vn[j]+256]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Unn[j]]+lookup[src_U[j]+256]+4)>>3);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_Vnn[j]]+lookup[src_V[j]+256]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Un[j]+512]+(unsigned short)src_Unnn[j]+4)>>3);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_Vn[j]+512]+(unsigned short)src_Vnnn[j]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_U[j]+512]+(unsigned short)src_Upp[j]+4)>>3);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_V[j]+512]+(unsigned short)src_Vpp[j]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Up[j]]+lookup[src_Un[j]+256]+4)>>3);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_Vp[j]]+lookup[src_Vn[j]+256]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_U,w_U);
		memcpy(dst_V,src_V,w_V);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_Un,w_U);
		memcpy(dst_V,src_Vn,w_V);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}

void JPSDR_AutoYUY2::Convert_Progressive_to_Planar422_SSE(const void *src_Y,const void *_src_U,const void *_src_V,void *dst_Y,void *dst_U,
		void *dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V)
{
	sint32 i,h_2,w_U8,w_V8;
	const unsigned char *src_U,*src_Up,*src_Un;
	const unsigned char *src_V,*src_Vp,*src_Vn;

	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vn=src_V+src_pitch_V;

	h_2=h_Y-2;
	w_U8=w_U>>3;
	w_V8=w_V>>3;

	JPSDR_AutoYUY2_Move32_Full(src_Y,dst_Y,w_Y>>2,h_Y,src_modulo_Y,dst_modulo_Y);

	for(i=0; i<2; i+=2)
	{
		memcpy(dst_U,src_U,w_U);
		memcpy(dst_V,src_V,w_V);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_4(src_U,src_Un,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_4(src_V,src_Vn,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=2; i<h_2; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_4(src_U,src_Up,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_4(src_V,src_Vp,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_4(src_U,src_Un,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_4(src_V,src_Vn,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=h_2; i<h_Y; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_4(src_U,src_Up,dst_U,w_U8);
		JPSDR_AutoYUY2_Convert420_to_Planar422_SSE2_4(src_V,src_Vp,dst_V,w_V8);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);


		memcpy(dst_U,src_U,w_U);
		memcpy(dst_V,src_V,w_V);
		dst_U=(void *)((char *)dst_U+dst_pitch_U);
		dst_V=(void *)((char *)dst_V+dst_pitch_V);

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}
}



void JPSDR_AutoYUY2::Convert_Progressive_to_Planar422_NonSSE(const void *src_Y,const void *_src_U,const void *_src_V,void *dst_Y,void *_dst_U,
		void *_dst_V,sint32 w_Y,sint32 w_U,sint32 w_V,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V)
{
	sint32 i,h_2,j;
	const unsigned char *src_U,*src_Up,*src_Un;
	const unsigned char *src_V,*src_Vp,*src_Vn;
	unsigned char *dst_U,*dst_V;

	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	dst_U=(unsigned char *)_dst_U;
	dst_V=(unsigned char *)_dst_V;

	h_2=h_Y-2;

	if ((w_Y&03)==0) JPSDR_AutoYUY2_Move32_Full(src_Y,dst_Y,w_Y>>2,h_Y,src_modulo_Y,dst_modulo_Y);
	else JPSDR_AutoYUY2_Move8_Full(src_Y,dst_Y,w_Y,h_Y,src_modulo_Y,dst_modulo_Y);

	for(i=0; i<2; i+=2)
	{
		memcpy(dst_U,src_U,w_U);
		memcpy(dst_V,src_V,w_V);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_U[j]]+(unsigned short)src_Un[j]+2)>>2);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_V[j]]+(unsigned short)src_Vn[j]+2)>>2);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=2; i<h_2; i+=2)
	{
		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_U[j]]+(unsigned short)src_Up[j]+2)>>2);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_V[j]]+(unsigned short)src_Vp[j]+2)>>2);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_U[j]]+(unsigned short)src_Un[j]+2)>>2);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_V[j]]+(unsigned short)src_Vn[j]+2)>>2);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=h_2; i<h_Y; i+=2)
	{
		for(j=0; j<w_U; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_U[j]]+(unsigned short)src_Up[j]+2)>>2);
		}
		for(j=0; j<w_V; j++)
		{
			dst_V[j]=(unsigned char)((lookup[src_V[j]]+(unsigned short)src_Vp[j]+2)>>2);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_U,w_U);
		memcpy(dst_V,src_V,w_V);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}
}


void JPSDR_AutoYUY2::Convert_Progressive_to_YUY2(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	if (SSE2_Enable)
		Convert_Progressive_to_YUY2_SSE(src_Y,src_U,src_V,dst,w_Y,h_Y,src_pitch_Y,src_pitch_U,src_pitch_V,dst_pitch);
	else
		Convert_Progressive_to_YUY2_NonSSE(src_Y,src_U,src_V,dst,w_Y,h_Y,src_pitch_Y,src_pitch_U,src_pitch_V,dst_pitch);
}


void JPSDR_AutoYUY2::Convert_Progressive_to_YUY2_SSE(const void *src_Y,const void *_src_U,const void *_src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	sint32 i,h_2,w_Y2,w_Y4;
	const unsigned char *src_U,*src_Up,*src_Un;
	const unsigned char *src_V,*src_Vp,*src_Vn;

	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vn=src_V+src_pitch_V;

	h_2=h_Y-2;
	w_Y4=w_Y>>2;
	w_Y2=w_Y>>1;

	for(i=0; i<2; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_4(src_Y,src_U,src_Un,src_V,src_Vn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=2; i<h_2; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_4(src_Y,src_U,src_Up,src_V,src_Vp,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_4(src_Y,src_U,src_Un,src_V,src_Vn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=h_2; i<h_Y; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_4(src_Y,src_U,src_Up,src_V,src_Vp,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}
}


void JPSDR_AutoYUY2::Convert_Progressive_to_YUY2_NonSSE(const void *_src_Y,const void *_src_U,const void *_src_V,void *_dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	sint32 i,h_2,j,j_UV,w_Y2;
	const unsigned char *src_Y;
	const unsigned char *src_U,*src_Up,*src_Un;
	const unsigned char *src_V,*src_Vp,*src_Vn;
	YUYV *dst;

	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vn=src_V+src_pitch_V;

	src_Y=(unsigned char *)_src_Y;
	dst=(YUYV *)_dst;

	h_2=h_Y-2;
	w_Y2=(w_Y+1)>>1;

	for(i=0; i<2; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Un[j_UV]+2)>>2);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vn[j_UV]+2)>>2);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=2; i<h_2; i+=2)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Up[j_UV]+2)>>2);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vp[j_UV]+2)>>2);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Un[j_UV]+2)>>2);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vn[j_UV]+2)>>2);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=h_2; i<h_Y; i+=2)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Up[j_UV]+2)>>2);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vp[j_UV]+2)>>2);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}
}


void JPSDR_AutoYUY2::Convert_Interlaced_to_YUY2(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	if (SSE2_Enable)
		Convert_Interlaced_to_YUY2_SSE(src_Y,src_U,src_V,dst,w_Y,h_Y,src_pitch_Y,src_pitch_U,src_pitch_V,dst_pitch);
	else
		Convert_Interlaced_to_YUY2_NonSSE(src_Y,src_U,src_V,dst,w_Y,h_Y,src_pitch_Y,src_pitch_U,src_pitch_V,dst_pitch);
}

void JPSDR_AutoYUY2::Convert_Interlaced_to_YUY2_SSE(const void *src_Y,const void *_src_U,const void *_src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	sint32 i,h_4,w_Y2,w_Y4;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	ptrdiff_t pitch_U_2,pitch_V_2;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	h_4=h_Y-4;
	w_Y2=w_Y>>1;
	w_Y4=w_Y>>2;

	for(i=0; i<4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_2(src_Y,src_Unn,src_U,src_Vnn,src_V,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_3(src_Y,src_Un,src_Unnn,src_Vn,src_Vnnn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=4; i<h_4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_3(src_Y,src_U,src_Upp,src_V,src_Vpp,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_2(src_Y,src_Up,src_Un,src_Vp,src_Vn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_2(src_Y,src_Unn,src_U,src_Vnn,src_V,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_3(src_Y,src_Un,src_Unnn,src_Vn,src_Vnnn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_3(src_Y,src_U,src_Upp,src_V,src_Vpp,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_SSE2_2(src_Y,src_Up,src_Un,src_Vp,src_Vn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}


void JPSDR_AutoYUY2::Convert_Interlaced_to_YUY2_NonSSE(const void *_src_Y,const void *_src_U,const void *_src_V,void *_dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	sint32 i,h_4,w_Y2,j,j_UV;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	const unsigned char *src_Y;
	ptrdiff_t pitch_U_2,pitch_V_2;
	YUYV *dst;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	src_Y=(unsigned char *)_src_Y;
	dst=(YUYV *)_dst;

	h_4=h_Y-4;
	w_Y2=(w_Y+1)>>1;

	for(i=0; i<4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=4; i<h_4; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}


void JPSDR_AutoYUY2::Convert_Progressive_to_UYVY(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	if (SSE2_Enable)
		Convert_Progressive_to_UYVY_SSE(src_Y,src_U,src_V,dst,w_Y,h_Y,src_pitch_Y,src_pitch_U,src_pitch_V,dst_pitch);
	else
		Convert_Progressive_to_UYVY_NonSSE(src_Y,src_U,src_V,dst,w_Y,h_Y,src_pitch_Y,src_pitch_U,src_pitch_V,dst_pitch);
}

void JPSDR_AutoYUY2::Convert_Progressive_to_UYVY_SSE(const void *src_Y,const void *_src_U,const void *_src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	sint32 i,h_2,w_Y2,w_Y4;
	const unsigned char *src_U,*src_Up,*src_Un;
	const unsigned char *src_V,*src_Vp,*src_Vn;

	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vn=src_V+src_pitch_V;

	h_2=h_Y-2;
	w_Y2=w_Y>>1;
	w_Y4=w_Y>>2;

	for(i=0; i<2; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_4(src_Y,src_U,src_Un,src_V,src_Vn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=2; i<h_2; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_4(src_Y,src_U,src_Up,src_V,src_Vp,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_4(src_Y,src_U,src_Un,src_V,src_Vn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=h_2; i<h_Y; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_4(src_Y,src_U,src_Up,src_V,src_Vp,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}
}


void JPSDR_AutoYUY2::Convert_Progressive_to_UYVY_NonSSE(const void *_src_Y,const void *_src_U,const void *_src_V,void *_dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	sint32 i,h_2,j,j_UV,w_Y2;
	const unsigned char *src_Y;
	const unsigned char *src_U,*src_Up,*src_Un;
	const unsigned char *src_V,*src_Vp,*src_Vn;
	UYVY *dst;

	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vn=src_V+src_pitch_V;

	src_Y=(unsigned char *)_src_Y;
	dst=(UYVY *)_dst;

	h_2=h_Y-2;
	w_Y2=(w_Y+1)>>1;

	for(i=0; i<2; i+=2)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Un[j_UV]+2)>>2);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vn[j_UV]+2)>>2);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=2; i<h_2; i+=2)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Up[j_UV]+2)>>2);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vp[j_UV]+2)>>2);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Un[j_UV]+2)>>2);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vn[j_UV]+2)>>2);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}

	for(i=h_2; i<h_Y; i+=2)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Up[j_UV]+2)>>2);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vp[j_UV]+2)>>2);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=src_pitch_U;
		src_Up+=src_pitch_U;
		src_Un+=src_pitch_U;

		src_V+=src_pitch_V;
		src_Vp+=src_pitch_V;
		src_Vn+=src_pitch_V;
	}
}


void JPSDR_AutoYUY2::Convert_Interlaced_to_UYVY(const void *src_Y,const void *src_U,const void *src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	if (SSE2_Enable)
		Convert_Interlaced_to_UYVY_SSE(src_Y,src_U,src_V,dst,w_Y,h_Y,src_pitch_Y,src_pitch_U,src_pitch_V,dst_pitch);
	else
		Convert_Interlaced_to_UYVY_NonSSE(src_Y,src_U,src_V,dst,w_Y,h_Y,src_pitch_Y,src_pitch_U,src_pitch_V,dst_pitch);
}


void JPSDR_AutoYUY2::Convert_Interlaced_to_UYVY_SSE(const void *src_Y,const void *_src_U,const void *_src_V,void *dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	sint32 i,h_4,w_Y2,w_Y4;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	ptrdiff_t pitch_U_2,pitch_V_2;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	h_4=h_Y-4;
	w_Y2=w_Y>>1;
	w_Y4=w_Y>>2;

	for(i=0; i<4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_2(src_Y,src_Unn,src_U,src_Vnn,src_V,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_3(src_Y,src_Un,src_Unnn,src_Vn,src_Vnnn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=4; i<h_4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_3(src_Y,src_U,src_Upp,src_V,src_Vpp,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_2(src_Y,src_Up,src_Un,src_Vp,src_Vn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_2(src_Y,src_Unn,src_U,src_Vnn,src_V,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_3(src_Y,src_Un,src_Unnn,src_Vn,src_Vnnn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_3(src_Y,src_U,src_Upp,src_V,src_Vpp,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_SSE2_2(src_Y,src_Up,src_Un,src_Vp,src_Vn,dst,w_Y4);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(void *)((char *)dst+dst_pitch);
		src_Y=(void *)((char *)src_Y+src_pitch_Y);

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}


void JPSDR_AutoYUY2::Convert_Interlaced_to_UYVY_NonSSE(const void *_src_Y,const void *_src_U,const void *_src_V,void *_dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch)
{
	sint32 i,h_4,w_Y2,j,j_UV;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	const unsigned char *src_Y;
	ptrdiff_t pitch_U_2,pitch_V_2;
	UYVY *dst;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	src_Y=(unsigned char *)_src_Y;
	dst=(UYVY *)_dst;

	h_4=h_Y-4;
	w_Y2=(w_Y+1)>>1;

	for(i=0; i<4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=4; i<h_4; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}



void JPSDR_AutoYUY2::Convert_Automatic_to_YUY2(const void *_src_Y,const void *_src_U,const void *_src_V,void *_dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch,
		unsigned char thrs)
{
	sint32 i,h_4,w_Y2,j,j_UV;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	const unsigned char *src_Y;
	signed short threshold;
	ptrdiff_t pitch_U_2,pitch_V_2;
	unsigned char index_tab_0,index_tab_1,index_tab_2;
	YUYV *dst;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	src_Y=(unsigned char *)_src_Y;
	dst=(YUYV *)_dst;
	threshold=thrs;

	h_4=h_Y-4;
	w_Y2=(w_Y+1)>>1;

	for(i=0; i<4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
	
			if (((abs((signed short)src_Un[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				(abs((signed short)src_Unnn[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				((src_Un[j_UV]>src_Unn[j_UV]) && (src_Unnn[j_UV]>src_Unn[j_UV]) ||
				(src_Un[j_UV]<src_Unn[j_UV]) && (src_Unnn[j_UV]<src_Unn[j_UV]))) ||
				((abs((signed short)src_Vn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnnn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				((src_Vn[j_UV]>src_Vnn[j_UV]) && (src_Vnnn[j_UV]>src_Vnn[j_UV]) ||
				(src_Vn[j_UV]<src_Vnn[j_UV]) && (src_Vnnn[j_UV]<src_Vnn[j_UV]))))
				mData.interlaced_tab[1][j_UV]=true;
			else mData.interlaced_tab[1][j_UV]=false;
			if (((abs((signed short)src_U[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				(abs((signed short)src_Unn[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				((src_U[j_UV]>src_Un[j_UV]) && (src_Unn[j_UV]>src_Un[j_UV]) ||
				(src_U[j_UV]<src_Un[j_UV]) && (src_Unn[j_UV]<src_Un[j_UV]))) ||
				((abs((signed short)src_V[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnn[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				((src_V[j_UV]>src_Vn[j_UV]) && (src_Vnn[j_UV]>src_Vn[j_UV]) ||
				(src_V[j_UV]<src_Vn[j_UV]) && (src_Vnn[j_UV]<src_Vn[j_UV]))))
				mData.interlaced_tab[0][j_UV]=true;
			else mData.interlaced_tab[0][j_UV]=false;

			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	index_tab_0=0;
	index_tab_1=1;
	index_tab_2=2;

	for(i=4; i<h_4; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_0][j_UV]))
			{
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4) >> 3);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4) >> 3);
			}
			else
			{
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Up[j_UV]+2) >> 2);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vp[j_UV]+2) >> 2);
			}
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];

			if (((abs((signed short)src_U[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				(abs((signed short)src_Unn[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				((src_U[j_UV]>src_Un[j_UV]) && (src_Unn[j_UV]>src_Un[j_UV]) ||
				(src_U[j_UV]<src_Un[j_UV]) && (src_Unn[j_UV]<src_Un[j_UV]))) ||
				((abs((signed short)src_V[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnn[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				((src_V[j_UV]>src_Vn[j_UV]) && (src_Vnn[j_UV]>src_Vn[j_UV]) ||
				(src_V[j_UV]<src_Vn[j_UV]) && (src_Vnn[j_UV]<src_Vn[j_UV]))))
				mData.interlaced_tab[index_tab_2][j_UV]=true;
			else mData.interlaced_tab[index_tab_2][j_UV]=false;			

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_2][j_UV]))
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
				dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			}
			else
			{
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Un[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vn[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_2][j_UV]))
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
				dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			}
			else
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]]+(unsigned short)src_U[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]]+(unsigned short)src_V[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];

			if (((abs((signed short)src_Un[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				(abs((signed short)src_Unnn[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				((src_Un[j_UV]>src_Unn[j_UV]) && (src_Unnn[j_UV]>src_Unn[j_UV]) ||
				(src_Un[j_UV]<src_Unn[j_UV]) && (src_Unnn[j_UV]<src_Unn[j_UV]))) ||
				((abs((signed short)src_Vn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnnn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				((src_Vn[j_UV]>src_Vnn[j_UV]) && (src_Vnnn[j_UV]>src_Vnn[j_UV]) ||
				(src_Vn[j_UV]<src_Vnn[j_UV]) && (src_Vnnn[j_UV]<src_Vnn[j_UV]))))
				mData.interlaced_tab[index_tab_0][j_UV]=true;
			else mData.interlaced_tab[index_tab_0][j_UV]=false;

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_2][j_UV]) || (mData.interlaced_tab[index_tab_0][j_UV]))
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
			}
			else
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]]+(unsigned short)src_Unn[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]]+(unsigned short)src_Vnn[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		index_tab_0=(index_tab_0+2)%Interlaced_Tab_Size;
		index_tab_1=(index_tab_1+2)%Interlaced_Tab_Size;
		index_tab_2=(index_tab_2+2)%Interlaced_Tab_Size;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}


void JPSDR_AutoYUY2::Convert_Test_to_YUY2(const void *_src_Y,const void *_src_U,const void *_src_V,void *_dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch,
		unsigned char thrs)
{
	sint32 i,h_4,w_Y2,j,j_UV;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	const unsigned char *src_Y;
	signed short threshold;
	ptrdiff_t pitch_U_2,pitch_V_2;
	unsigned char index_tab_0,index_tab_1,index_tab_2;
	YUYV *dst;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	src_Y=(unsigned char *)_src_Y;
	dst=(YUYV *)_dst;
	threshold=thrs;

	h_4=h_Y-4;
	w_Y2=(w_Y+1)>>1;

	for(i=0; i<4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
	
			if (((abs((signed short)src_Un[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				(abs((signed short)src_Unnn[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				((src_Un[j_UV]>src_Unn[j_UV]) && (src_Unnn[j_UV]>src_Unn[j_UV]) ||
				(src_Un[j_UV]<src_Unn[j_UV]) && (src_Unnn[j_UV]<src_Unn[j_UV]))) ||
				((abs((signed short)src_Vn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnnn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				((src_Vn[j_UV]>src_Vnn[j_UV]) && (src_Vnnn[j_UV]>src_Vnn[j_UV]) ||
				(src_Vn[j_UV]<src_Vnn[j_UV]) && (src_Vnnn[j_UV]<src_Vnn[j_UV]))))
				mData.interlaced_tab[1][j_UV]=true;
			else mData.interlaced_tab[1][j_UV]=false;
			if (((abs((signed short)src_U[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				(abs((signed short)src_Unn[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				((src_U[j_UV]>src_Un[j_UV]) && (src_Unn[j_UV]>src_Un[j_UV]) ||
				(src_U[j_UV]<src_Un[j_UV]) && (src_Unn[j_UV]<src_Un[j_UV]))) ||
				((abs((signed short)src_V[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnn[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				((src_V[j_UV]>src_Vn[j_UV]) && (src_Vnn[j_UV]>src_Vn[j_UV]) ||
				(src_V[j_UV]<src_Vn[j_UV]) && (src_Vnn[j_UV]<src_Vn[j_UV]))))
				mData.interlaced_tab[0][j_UV]=true;
			else mData.interlaced_tab[0][j_UV]=false;

			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	index_tab_0=0;
	index_tab_1=1;
	index_tab_2=2;

	for(i=4; i<h_4; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_0][j_UV]))
			{
				dst[j_UV].y1=180;
				dst[j_UV].y2=180;
				dst[j_UV].u=128;
				dst[j_UV].v=128;
			}
			else
			{
				dst[j_UV].y1=src_Y[j];
				dst[j_UV].y2=src_Y[j+1];
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Up[j_UV]+2) >> 2);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vp[j_UV]+2) >> 2);
			}
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			if (((abs((signed short)src_U[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				(abs((signed short)src_Unn[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				((src_U[j_UV]>src_Un[j_UV]) && (src_Unn[j_UV]>src_Un[j_UV]) ||
				(src_U[j_UV]<src_Un[j_UV]) && (src_Unn[j_UV]<src_Un[j_UV]))) ||
				((abs((signed short)src_V[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnn[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				((src_V[j_UV]>src_Vn[j_UV]) && (src_Vnn[j_UV]>src_Vn[j_UV]) ||
				(src_V[j_UV]<src_Vn[j_UV]) && (src_Vnn[j_UV]<src_Vn[j_UV]))))
				mData.interlaced_tab[index_tab_2][j_UV]=true;
			else mData.interlaced_tab[index_tab_2][j_UV]=false;			

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_2][j_UV]))
			{
				dst[j_UV].y1=180;
				dst[j_UV].y2=180;
				dst[j_UV].u=128;
				dst[j_UV].v=128;
			}
			else
			{
				dst[j_UV].y1=src_Y[j];
				dst[j_UV].y2=src_Y[j+1];
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Un[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vn[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_2][j_UV]))
			{
				dst[j_UV].y1=180;
				dst[j_UV].y2=180;
				dst[j_UV].u=128;
				dst[j_UV].v=128;
			}
			else
			{
				dst[j_UV].y1=src_Y[j];
				dst[j_UV].y2=src_Y[j+1];
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]]+(unsigned short)src_U[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]]+(unsigned short)src_V[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			if (((abs((signed short)src_Un[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				(abs((signed short)src_Unnn[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				((src_Un[j_UV]>src_Unn[j_UV]) && (src_Unnn[j_UV]>src_Unn[j_UV]) ||
				(src_Un[j_UV]<src_Unn[j_UV]) && (src_Unnn[j_UV]<src_Unn[j_UV]))) ||
				((abs((signed short)src_Vn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnnn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				((src_Vn[j_UV]>src_Vnn[j_UV]) && (src_Vnnn[j_UV]>src_Vnn[j_UV]) ||
				(src_Vn[j_UV]<src_Vnn[j_UV]) && (src_Vnnn[j_UV]<src_Vnn[j_UV]))))
				mData.interlaced_tab[index_tab_0][j_UV]=true;
			else mData.interlaced_tab[index_tab_0][j_UV]=false;

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_2][j_UV]) || (mData.interlaced_tab[index_tab_0][j_UV]))
			{
				dst[j_UV].y1=180;
				dst[j_UV].y2=180;
				dst[j_UV].u=128;
				dst[j_UV].v=128;
			}
			else
			{
				dst[j_UV].y1=src_Y[j];
				dst[j_UV].y2=src_Y[j+1];
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]]+(unsigned short)src_Unn[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]]+(unsigned short)src_Vnn[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		index_tab_0=(index_tab_0+2)%Interlaced_Tab_Size;
		index_tab_1=(index_tab_1+2)%Interlaced_Tab_Size;
		index_tab_2=(index_tab_2+2)%Interlaced_Tab_Size;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_YUY2_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(YUYV *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}



void JPSDR_AutoYUY2::Convert_Automatic_to_UYVY(const void *_src_Y,const void *_src_U,const void *_src_V,void *_dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch,
		unsigned char thrs)
{
	sint32 i,h_4,w_Y2,j,j_UV;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	const unsigned char *src_Y;
	signed short threshold;
	ptrdiff_t pitch_U_2,pitch_V_2;
	unsigned char index_tab_0,index_tab_1,index_tab_2;
	UYVY *dst;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	src_Y=(unsigned char *)_src_Y;
	dst=(UYVY *)_dst;
	threshold=thrs;

	h_4=h_Y-4;
	w_Y2=(w_Y+1)>>1;

	for(i=0; i<4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
	
			if (((abs((signed short)src_Un[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				(abs((signed short)src_Unnn[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				((src_Un[j_UV]>src_Unn[j_UV]) && (src_Unnn[j_UV]>src_Unn[j_UV]) ||
				(src_Un[j_UV]<src_Unn[j_UV]) && (src_Unnn[j_UV]<src_Unn[j_UV]))) ||
				((abs((signed short)src_Vn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnnn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				((src_Vn[j_UV]>src_Vnn[j_UV]) && (src_Vnnn[j_UV]>src_Vnn[j_UV]) ||
				(src_Vn[j_UV]<src_Vnn[j_UV]) && (src_Vnnn[j_UV]<src_Vnn[j_UV]))))
				mData.interlaced_tab[1][j_UV]=true;
			else mData.interlaced_tab[1][j_UV]=false;
			if (((abs((signed short)src_U[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				(abs((signed short)src_Unn[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				((src_U[j_UV]>src_Un[j_UV]) && (src_Unn[j_UV]>src_Un[j_UV]) ||
				(src_U[j_UV]<src_Un[j_UV]) && (src_Unn[j_UV]<src_Un[j_UV]))) ||
				((abs((signed short)src_V[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnn[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				((src_V[j_UV]>src_Vn[j_UV]) && (src_Vnn[j_UV]>src_Vn[j_UV]) ||
				(src_V[j_UV]<src_Vn[j_UV]) && (src_Vnn[j_UV]<src_Vn[j_UV]))))
				mData.interlaced_tab[0][j_UV]=true;
			else mData.interlaced_tab[0][j_UV]=false;

			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	index_tab_0=0;
	index_tab_1=1;
	index_tab_2=2;

	for(i=4; i<h_4; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_0][j_UV]))
			{
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4) >> 3);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4) >> 3);
			}
			else
			{
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Up[j_UV]+2) >> 2);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vp[j_UV]+2) >> 2);
			}
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];

			if (((abs((signed short)src_U[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				(abs((signed short)src_Unn[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				((src_U[j_UV]>src_Un[j_UV]) && (src_Unn[j_UV]>src_Un[j_UV]) ||
				(src_U[j_UV]<src_Un[j_UV]) && (src_Unn[j_UV]<src_Un[j_UV]))) ||
				((abs((signed short)src_V[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnn[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				((src_V[j_UV]>src_Vn[j_UV]) && (src_Vnn[j_UV]>src_Vn[j_UV]) ||
				(src_V[j_UV]<src_Vn[j_UV]) && (src_Vnn[j_UV]<src_Vn[j_UV]))))
				mData.interlaced_tab[index_tab_2][j_UV]=true;
			else mData.interlaced_tab[index_tab_2][j_UV]=false;			

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_2][j_UV]))
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
				dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			}
			else
			{
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Un[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vn[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_2][j_UV]))
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
				dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			}
			else
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]]+(unsigned short)src_U[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]]+(unsigned short)src_V[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];

			if (((abs((signed short)src_Un[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				(abs((signed short)src_Unnn[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				((src_Un[j_UV]>src_Unn[j_UV]) && (src_Unnn[j_UV]>src_Unn[j_UV]) ||
				(src_Un[j_UV]<src_Unn[j_UV]) && (src_Unnn[j_UV]<src_Unn[j_UV]))) ||
				((abs((signed short)src_Vn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnnn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				((src_Vn[j_UV]>src_Vnn[j_UV]) && (src_Vnnn[j_UV]>src_Vnn[j_UV]) ||
				(src_Vn[j_UV]<src_Vnn[j_UV]) && (src_Vnnn[j_UV]<src_Vnn[j_UV]))))
				mData.interlaced_tab[index_tab_0][j_UV]=true;
			else mData.interlaced_tab[index_tab_0][j_UV]=false;

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_2][j_UV]) || (mData.interlaced_tab[index_tab_0][j_UV]))
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
			}
			else
			{
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]]+(unsigned short)src_Unn[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]]+(unsigned short)src_Vnn[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		index_tab_0=(index_tab_0+2)%Interlaced_Tab_Size;
		index_tab_1=(index_tab_1+2)%Interlaced_Tab_Size;
		index_tab_2=(index_tab_2+2)%Interlaced_Tab_Size;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}


void JPSDR_AutoYUY2::Convert_Test_to_UYVY(const void *_src_Y,const void *_src_U,const void *_src_V,void *_dst,
		sint32 w_Y,sint32 h_Y,ptrdiff_t src_pitch_Y,ptrdiff_t src_pitch_U,ptrdiff_t src_pitch_V,ptrdiff_t dst_pitch,
		unsigned char thrs)
{
	sint32 i,h_4,w_Y2,j,j_UV;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	const unsigned char *src_Y;
	signed short threshold;
	ptrdiff_t pitch_U_2,pitch_V_2;
	unsigned char index_tab_0,index_tab_1,index_tab_2;
	UYVY *dst;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	src_Y=(unsigned char *)_src_Y;
	dst=(UYVY *)_dst;
	threshold=thrs;

	h_4=h_Y-4;
	w_Y2=(w_Y+1)>>1;

	for(i=0; i<4; i+=4)
	{
		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Unn[j_UV]]+lookup[src_U[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vnn[j_UV]]+lookup[src_V[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]+512]+(unsigned short)src_Unnn[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]+512]+(unsigned short)src_Vnnn[j_UV]+4)>>3);
	
			if (((abs((signed short)src_Un[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				(abs((signed short)src_Unnn[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				((src_Un[j_UV]>src_Unn[j_UV]) && (src_Unnn[j_UV]>src_Unn[j_UV]) ||
				(src_Un[j_UV]<src_Unn[j_UV]) && (src_Unnn[j_UV]<src_Unn[j_UV]))) ||
				((abs((signed short)src_Vn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnnn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				((src_Vn[j_UV]>src_Vnn[j_UV]) && (src_Vnnn[j_UV]>src_Vnn[j_UV]) ||
				(src_Vn[j_UV]<src_Vnn[j_UV]) && (src_Vnnn[j_UV]<src_Vnn[j_UV]))))
				mData.interlaced_tab[1][j_UV]=true;
			else mData.interlaced_tab[1][j_UV]=false;
			if (((abs((signed short)src_U[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				(abs((signed short)src_Unn[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				((src_U[j_UV]>src_Un[j_UV]) && (src_Unn[j_UV]>src_Un[j_UV]) ||
				(src_U[j_UV]<src_Un[j_UV]) && (src_Unn[j_UV]<src_Un[j_UV]))) ||
				((abs((signed short)src_V[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnn[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				((src_V[j_UV]>src_Vn[j_UV]) && (src_Vnn[j_UV]>src_Vn[j_UV]) ||
				(src_V[j_UV]<src_Vn[j_UV]) && (src_Vnn[j_UV]<src_Vn[j_UV]))))
				mData.interlaced_tab[0][j_UV]=true;
			else mData.interlaced_tab[0][j_UV]=false;

			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	index_tab_0=0;
	index_tab_1=1;
	index_tab_2=2;

	for(i=4; i<h_4; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_0][j_UV]))
			{
				dst[j_UV].y1=180;
				dst[j_UV].y2=180;
				dst[j_UV].u=128;
				dst[j_UV].v=128;
			}
			else
			{
				dst[j_UV].y1=src_Y[j];
				dst[j_UV].y2=src_Y[j+1];
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Up[j_UV]+2) >> 2);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vp[j_UV]+2) >> 2);
			}
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			if (((abs((signed short)src_U[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				(abs((signed short)src_Unn[j_UV]-(signed short)src_Un[j_UV])>=threshold) &&
				((src_U[j_UV]>src_Un[j_UV]) && (src_Unn[j_UV]>src_Un[j_UV]) ||
				(src_U[j_UV]<src_Un[j_UV]) && (src_Unn[j_UV]<src_Un[j_UV]))) ||
				((abs((signed short)src_V[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnn[j_UV]-(signed short)src_Vn[j_UV])>=threshold) &&
				((src_V[j_UV]>src_Vn[j_UV]) && (src_Vnn[j_UV]>src_Vn[j_UV]) ||
				(src_V[j_UV]<src_Vn[j_UV]) && (src_Vnn[j_UV]<src_Vn[j_UV]))))
				mData.interlaced_tab[index_tab_2][j_UV]=true;
			else mData.interlaced_tab[index_tab_2][j_UV]=false;			

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_2][j_UV]))
			{
				dst[j_UV].y1=180;
				dst[j_UV].y2=180;
				dst[j_UV].u=128;
				dst[j_UV].v=128;
			}
			else
			{
				dst[j_UV].y1=src_Y[j];
				dst[j_UV].y2=src_Y[j+1];
				dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]]+(unsigned short)src_Un[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]]+(unsigned short)src_Vn[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j_UV]) || (mData.interlaced_tab[index_tab_2][j_UV]))
			{
				dst[j_UV].y1=180;
				dst[j_UV].y2=180;
				dst[j_UV].u=128;
				dst[j_UV].v=128;
			}
			else
			{
				dst[j_UV].y1=src_Y[j];
				dst[j_UV].y2=src_Y[j+1];
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]]+(unsigned short)src_U[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]]+(unsigned short)src_V[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			if (((abs((signed short)src_Un[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				(abs((signed short)src_Unnn[j_UV]-(signed short)src_Unn[j_UV])>=threshold) &&
				((src_Un[j_UV]>src_Unn[j_UV]) && (src_Unnn[j_UV]>src_Unn[j_UV]) ||
				(src_Un[j_UV]<src_Unn[j_UV]) && (src_Unnn[j_UV]<src_Unn[j_UV]))) ||
				((abs((signed short)src_Vn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				(abs((signed short)src_Vnnn[j_UV]-(signed short)src_Vnn[j_UV])>=threshold) &&
				((src_Vn[j_UV]>src_Vnn[j_UV]) && (src_Vnnn[j_UV]>src_Vnn[j_UV]) ||
				(src_Vn[j_UV]<src_Vnn[j_UV]) && (src_Vnnn[j_UV]<src_Vnn[j_UV]))))
				mData.interlaced_tab[index_tab_0][j_UV]=true;
			else mData.interlaced_tab[index_tab_0][j_UV]=false;

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_2][j_UV]) || (mData.interlaced_tab[index_tab_0][j_UV]))
			{
				dst[j_UV].y1=180;
				dst[j_UV].y2=180;
				dst[j_UV].u=128;
				dst[j_UV].v=128;
			}
			else
			{
				dst[j_UV].y1=src_Y[j];
				dst[j_UV].y2=src_Y[j+1];
				dst[j_UV].u=(unsigned char)((lookup[src_Un[j_UV]]+(unsigned short)src_Unn[j_UV]+2)>>2);
				dst[j_UV].v=(unsigned char)((lookup[src_Vn[j_UV]]+(unsigned short)src_Vnn[j_UV]+2)>>2);
			}
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		index_tab_0=(index_tab_0+2)%Interlaced_Tab_Size;
		index_tab_1=(index_tab_1+2)%Interlaced_Tab_Size;
		index_tab_2=(index_tab_2+2)%Interlaced_Tab_Size;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_U[j_UV]+512]+(unsigned short)src_Upp[j_UV]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_V[j_UV]+512]+(unsigned short)src_Vpp[j_UV]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		j_UV=0;
		for (j=0; j<w_Y; j+=2)
		{
			dst[j_UV].y1=src_Y[j];
			dst[j_UV].y2=src_Y[j+1];
			dst[j_UV].u=(unsigned char)((lookup[src_Up[j_UV]]+lookup[src_Un[j_UV]+256]+4)>>3);
			dst[j_UV].v=(unsigned char)((lookup[src_Vp[j_UV]]+lookup[src_Vn[j_UV]+256]+4)>>3);
			j_UV++;
		}
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_U,src_V,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		JPSDR_AutoYUY2_Convert420_to_UYVY_1(src_Y,src_Un,src_Vn,dst,w_Y2);
		dst=(UYVY *)((char *)dst+dst_pitch);
		src_Y+=src_pitch_Y;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}
}


void JPSDR_AutoYUY2::Convert_Automatic_to_Planar422(const void *src_Y,const void *_src_U,const void *_src_V,void *dst_Y,
		void *_dst_U,void *_dst_V,sint32 w_Y,sint32 w_UV,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V,unsigned char thrs)
{
	sint32 i,h_4,j;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	unsigned char *dst_U,*dst_V;
	signed short threshold;
	ptrdiff_t pitch_U_2,pitch_V_2;
	unsigned char index_tab_0,index_tab_1,index_tab_2;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	threshold=thrs;
	dst_U=(unsigned char*)_dst_U;
	dst_V=(unsigned char*)_dst_V;

	h_4=h_Y-4;

	if ((w_Y&03)==0) JPSDR_AutoYUY2_Move32_Full(src_Y,dst_Y,w_Y>>2,h_Y,src_modulo_Y,dst_modulo_Y);
	else JPSDR_AutoYUY2_Move8_Full(src_Y,dst_Y,w_Y,h_Y,src_modulo_Y,dst_modulo_Y);

	for(i=0; i<4; i+=4)
	{
		memcpy(dst_U,src_U,w_UV);
		memcpy(dst_V,src_V,w_UV);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_Un,w_UV);
		memcpy(dst_V,src_Vn,w_UV);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Unn[j]]+lookup[src_U[j]+256]+4)>>3);
			dst_V[j]=(unsigned char)((lookup[src_Vnn[j]]+lookup[src_V[j]+256]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			if (((abs((signed short)src_Un[j]-(signed short)src_Unn[j])>=threshold) &&
				(abs((signed short)src_Unnn[j]-(signed short)src_Unn[j])>=threshold) &&
				((src_Un[j]>src_Unn[j]) && (src_Unnn[j]>src_Unn[j]) ||
				(src_Un[j]<src_Unn[j]) && (src_Unnn[j]<src_Unn[j]))) ||
				((abs((signed short)src_Vn[j]-(signed short)src_Vnn[j])>=threshold) &&
				(abs((signed short)src_Vnnn[j]-(signed short)src_Vnn[j])>=threshold) &&
				((src_Vn[j]>src_Vnn[j]) && (src_Vnnn[j]>src_Vnn[j]) ||
				(src_Vn[j]<src_Vnn[j]) && (src_Vnnn[j]<src_Vnn[j]))))
				mData.interlaced_tab[1][j]=true;
			else mData.interlaced_tab[1][j]=false;
			if (((abs((signed short)src_U[j]-(signed short)src_Un[j])>=threshold) &&
				(abs((signed short)src_Unn[j]-(signed short)src_Un[j])>=threshold) &&
				((src_U[j]>src_Un[j]) && (src_Unn[j]>src_Un[j]) ||
				(src_U[j]<src_Un[j]) && (src_Unn[j]<src_Un[j]))) ||
				((abs((signed short)src_V[j]-(signed short)src_Vn[j])>=threshold) &&
				(abs((signed short)src_Vnn[j]-(signed short)src_Vn[j])>=threshold) &&
				((src_V[j]>src_Vn[j]) && (src_Vnn[j]>src_Vn[j]) ||
				(src_V[j]<src_Vn[j]) && (src_Vnn[j]<src_Vn[j]))))
				mData.interlaced_tab[0][j]=true;
			else mData.interlaced_tab[0][j]=false;

			dst_U[j]=(unsigned char)((lookup[src_Un[j]+512]+(unsigned short)src_Unnn[j]+4)>>3);
			dst_V[j]=(unsigned char)((lookup[src_Vn[j]+512]+(unsigned short)src_Vnnn[j]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	index_tab_0=0;
	index_tab_1=1;
	index_tab_2=2;

	for(i=4; i<h_4; i+=4)
	{
		for(j=0; j<w_UV; j++)
		{
			if ((mData.interlaced_tab[index_tab_1][j]) || (mData.interlaced_tab[index_tab_0][j]))
			{
				dst_U[j]=(unsigned char)((lookup[src_U[j]+512]+(unsigned short)src_Upp[j]+4)>>3);
				dst_V[j]=(unsigned char)((lookup[src_V[j]+512]+(unsigned short)src_Vpp[j]+4)>>3);
			}
			else
			{
				dst_U[j]=(unsigned char)((lookup[src_U[j]]+(unsigned short)src_Up[j]+2) >> 2);
				dst_V[j]=(unsigned char)((lookup[src_V[j]]+(unsigned short)src_Vp[j]+2) >> 2);
			}
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			if (((abs((signed short)src_U[j]-(signed short)src_Un[j])>=threshold) &&
				(abs((signed short)src_Unn[j]-(signed short)src_Un[j])>=threshold) &&
				((src_U[j]>src_Un[j]) && (src_Unn[j]>src_Un[j]) ||
				(src_U[j]<src_Un[j]) && (src_Unn[j]<src_Un[j]))) ||
				((abs((signed short)src_V[j]-(signed short)src_Vn[j])>=threshold) &&
				(abs((signed short)src_Vnn[j]-(signed short)src_Vn[j])>=threshold) &&
				((src_V[j]>src_Vn[j]) && (src_Vnn[j]>src_Vn[j]) ||
				(src_V[j]<src_Vn[j]) && (src_Vnn[j]<src_Vn[j]))))
				mData.interlaced_tab[index_tab_2][j]=true;
			else mData.interlaced_tab[index_tab_2][j]=false;			

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j]) || (mData.interlaced_tab[index_tab_2][j]))
			{
				dst_U[j]=(unsigned char)((lookup[src_Up[j]]+lookup[src_Un[j]+256]+4)>>3);
				dst_V[j]=(unsigned char)((lookup[src_Vp[j]]+lookup[src_Vn[j]+256]+4)>>3);
			}
			else
			{
				dst_U[j]=(unsigned char)((lookup[src_U[j]]+(unsigned short)src_Un[j]+2)>>2);
				dst_V[j]=(unsigned char)((lookup[src_V[j]]+(unsigned short)src_Vn[j]+2)>>2);
			}
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			if ((mData.interlaced_tab[index_tab_1][j]) || (mData.interlaced_tab[index_tab_2][j]))
			{
				dst_U[j]=(unsigned char)((lookup[src_Unn[j]]+lookup[src_U[j]+256]+4)>>3);
				dst_V[j]=(unsigned char)((lookup[src_Vnn[j]]+lookup[src_V[j]+256]+4)>>3);
			}
			else
			{
				dst_U[j]=(unsigned char)((lookup[src_Un[j]]+(unsigned short)src_U[j]+2)>>2);
				dst_V[j]=(unsigned char)((lookup[src_Vn[j]]+(unsigned short)src_V[j]+2)>>2);
			}
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			if (((abs((signed short)src_Un[j]-(signed short)src_Unn[j])>=threshold) &&
				(abs((signed short)src_Unnn[j]-(signed short)src_Unn[j])>=threshold) &&
				((src_Un[j]>src_Unn[j]) && (src_Unnn[j]>src_Unn[j]) ||
				(src_Un[j]<src_Unn[j]) && (src_Unnn[j]<src_Unn[j]))) ||
				((abs((signed short)src_Vn[j]-(signed short)src_Vnn[j])>=threshold) &&
				(abs((signed short)src_Vnnn[j]-(signed short)src_Vnn[j])>=threshold) &&
				((src_Vn[j]>src_Vnn[j]) && (src_Vnnn[j]>src_Vnn[j]) ||
				(src_Vn[j]<src_Vnn[j]) && (src_Vnnn[j]<src_Vnn[j]))))
				mData.interlaced_tab[index_tab_0][j]=true;
			else mData.interlaced_tab[index_tab_0][j]=false;

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_2][j]) || (mData.interlaced_tab[index_tab_0][j]))
			{
				dst_U[j]=(unsigned char)((lookup[src_Un[j]+512]+(unsigned short)src_Unnn[j]+4)>>3);
				dst_V[j]=(unsigned char)((lookup[src_Vn[j]+512]+(unsigned short)src_Vnnn[j]+4)>>3);
			}
			else
			{
				dst_U[j]=(unsigned char)((lookup[src_Un[j]]+(unsigned short)src_Unn[j]+2)>>2);
				dst_V[j]=(unsigned char)((lookup[src_Vn[j]]+(unsigned short)src_Vnn[j]+2)>>2);
			}
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		index_tab_0=(index_tab_0+2)%Interlaced_Tab_Size;
		index_tab_1=(index_tab_1+2)%Interlaced_Tab_Size;
		index_tab_2=(index_tab_2+2)%Interlaced_Tab_Size;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		for(j=0; j<w_UV; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_U[j]+512]+(unsigned short)src_Upp[j]+4)>>3);
			dst_V[j]=(unsigned char)((lookup[src_V[j]+512]+(unsigned short)src_Vpp[j]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Up[j]]+lookup[src_Un[j]+256]+4)>>3);
			dst_V[j]=(unsigned char)((lookup[src_Vp[j]]+lookup[src_Vn[j]+256]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_U,w_UV);
		memcpy(dst_V,src_V,w_UV);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_Un,w_UV);
		memcpy(dst_V,src_Vn,w_UV);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

}


void JPSDR_AutoYUY2::Convert_Test_to_Planar422(const void *src_Y,const void *_src_U,const void *_src_V,void *dst_Y,
		void *_dst_U,void *_dst_V,sint32 w_Y,sint32 w_UV,sint32 h_Y,ptrdiff_t src_modulo_Y,ptrdiff_t src_pitch_U,
		ptrdiff_t src_pitch_V,ptrdiff_t dst_modulo_Y,ptrdiff_t dst_pitch_U,ptrdiff_t dst_pitch_V,unsigned char thrs)
{
	sint32 i,h_4,j;
	const unsigned char *src_U,*src_Up,*src_Upp,*src_Un,*src_Unn,*src_Unnn;
	const unsigned char *src_V,*src_Vp,*src_Vpp,*src_Vn,*src_Vnn,*src_Vnnn;
	unsigned char *dst_U,*dst_V;
	signed short threshold;
	ptrdiff_t pitch_U_2,pitch_V_2;
	unsigned char index_tab_0,index_tab_1,index_tab_2;

	pitch_U_2=2*src_pitch_U;
	pitch_V_2=2*src_pitch_V;
	src_U=(unsigned char *)_src_U;
	src_V=(unsigned char *)_src_V;
	src_Up=src_U-src_pitch_U;
	src_Upp=src_U-2*src_pitch_U;
	src_Un=src_U+src_pitch_U;
	src_Unn=src_U+2*src_pitch_U;
	src_Unnn=src_U+3*src_pitch_U;
	src_Vp=src_V-src_pitch_V;
	src_Vpp=src_V-2*src_pitch_V;
	src_Vn=src_V+src_pitch_V;
	src_Vnn=src_V+2*src_pitch_V;
	src_Vnnn=src_V+3*src_pitch_V;
	threshold=thrs;
	dst_U=(unsigned char*)_dst_U;
	dst_V=(unsigned char*)_dst_V;

	h_4=h_Y-4;

	if ((w_Y&03)==0) JPSDR_AutoYUY2_Move32_Full(src_Y,dst_Y,w_Y>>2,h_Y,src_modulo_Y,dst_modulo_Y);
	else JPSDR_AutoYUY2_Move8_Full(src_Y,dst_Y,w_Y,h_Y,src_modulo_Y,dst_modulo_Y);

	for(i=0; i<4; i+=4)
	{
		memcpy(dst_U,src_U,w_UV);
		memcpy(dst_V,src_V,w_UV);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_Un,w_UV);
		memcpy(dst_V,src_Vn,w_UV);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Unn[j]]+lookup[src_U[j]+256]+4)>>3);
			dst_V[j]=(unsigned char)((lookup[src_Vnn[j]]+lookup[src_V[j]+256]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			if (((abs((signed short)src_Un[j]-(signed short)src_Unn[j])>=threshold) &&
				(abs((signed short)src_Unnn[j]-(signed short)src_Unn[j])>=threshold) &&
				((src_Un[j]>src_Unn[j]) && (src_Unnn[j]>src_Unn[j]) ||
				(src_Un[j]<src_Unn[j]) && (src_Unnn[j]<src_Unn[j]))) ||
				((abs((signed short)src_Vn[j]-(signed short)src_Vnn[j])>=threshold) &&
				(abs((signed short)src_Vnnn[j]-(signed short)src_Vnn[j])>=threshold) &&
				((src_Vn[j]>src_Vnn[j]) && (src_Vnnn[j]>src_Vnn[j]) ||
				(src_Vn[j]<src_Vnn[j]) && (src_Vnnn[j]<src_Vnn[j]))))
				mData.interlaced_tab[1][j]=true;
			else mData.interlaced_tab[1][j]=false;
			if (((abs((signed short)src_U[j]-(signed short)src_Un[j])>=threshold) &&
				(abs((signed short)src_Unn[j]-(signed short)src_Un[j])>=threshold) &&
				((src_U[j]>src_Un[j]) && (src_Unn[j]>src_Un[j]) ||
				(src_U[j]<src_Un[j]) && (src_Unn[j]<src_Un[j]))) ||
				((abs((signed short)src_V[j]-(signed short)src_Vn[j])>=threshold) &&
				(abs((signed short)src_Vnn[j]-(signed short)src_Vn[j])>=threshold) &&
				((src_V[j]>src_Vn[j]) && (src_Vnn[j]>src_Vn[j]) ||
				(src_V[j]<src_Vn[j]) && (src_Vnn[j]<src_Vn[j]))))
				mData.interlaced_tab[0][j]=true;
			else mData.interlaced_tab[0][j]=false;

			dst_U[j]=(unsigned char)((lookup[src_Un[j]+512]+(unsigned short)src_Unnn[j]+4)>>3);
			dst_V[j]=(unsigned char)((lookup[src_Vn[j]+512]+(unsigned short)src_Vnnn[j]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	index_tab_0=0;
	index_tab_1=1;
	index_tab_2=2;

	for(i=4; i<h_4; i+=4)
	{
		for(j=0; j<w_UV; j++)
		{
			if ((mData.interlaced_tab[index_tab_1][j]) || (mData.interlaced_tab[index_tab_0][j]))
			{
				dst_U[j]=239;
				dst_V[j]=239;
			}
			else
			{
				dst_U[j]=(unsigned char)((lookup[src_U[j]]+(unsigned short)src_Up[j]+2) >> 2);
				dst_V[j]=(unsigned char)((lookup[src_V[j]]+(unsigned short)src_Vp[j]+2) >> 2);
			}
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			if (((abs((signed short)src_U[j]-(signed short)src_Un[j])>=threshold) &&
				(abs((signed short)src_Unn[j]-(signed short)src_Un[j])>=threshold) &&
				((src_U[j]>src_Un[j]) && (src_Unn[j]>src_Un[j]) ||
				(src_U[j]<src_Un[j]) && (src_Unn[j]<src_Un[j]))) ||
				((abs((signed short)src_V[j]-(signed short)src_Vn[j])>=threshold) &&
				(abs((signed short)src_Vnn[j]-(signed short)src_Vn[j])>=threshold) &&
				((src_V[j]>src_Vn[j]) && (src_Vnn[j]>src_Vn[j]) ||
				(src_V[j]<src_Vn[j]) && (src_Vnn[j]<src_Vn[j]))))
				mData.interlaced_tab[index_tab_2][j]=true;
			else mData.interlaced_tab[index_tab_2][j]=false;			

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_1][j]) || (mData.interlaced_tab[index_tab_2][j]))
			{
				dst_U[j]=239;
				dst_V[j]=239;
			}
			else
			{
				dst_U[j]=(unsigned char)((lookup[src_U[j]]+(unsigned short)src_Un[j]+2)>>2);
				dst_V[j]=(unsigned char)((lookup[src_V[j]]+(unsigned short)src_Vn[j]+2)>>2);
			}
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			if ((mData.interlaced_tab[index_tab_1][j]) || (mData.interlaced_tab[index_tab_2][j]))
			{
				dst_U[j]=239;
				dst_V[j]=239;
			}
			else
			{
				dst_U[j]=(unsigned char)((lookup[src_Un[j]]+(unsigned short)src_U[j]+2)>>2);
				dst_V[j]=(unsigned char)((lookup[src_Vn[j]]+(unsigned short)src_V[j]+2)>>2);
			}
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			if (((abs((signed short)src_Un[j]-(signed short)src_Unn[j])>=threshold) &&
				(abs((signed short)src_Unnn[j]-(signed short)src_Unn[j])>=threshold) &&
				((src_Un[j]>src_Unn[j]) && (src_Unnn[j]>src_Unn[j]) ||
				(src_Un[j]<src_Unn[j]) && (src_Unnn[j]<src_Unn[j]))) ||
				((abs((signed short)src_Vn[j]-(signed short)src_Vnn[j])>=threshold) &&
				(abs((signed short)src_Vnnn[j]-(signed short)src_Vnn[j])>=threshold) &&
				((src_Vn[j]>src_Vnn[j]) && (src_Vnnn[j]>src_Vnn[j]) ||
				(src_Vn[j]<src_Vnn[j]) && (src_Vnnn[j]<src_Vnn[j]))))
				mData.interlaced_tab[index_tab_0][j]=true;
			else mData.interlaced_tab[index_tab_0][j]=false;

			// Upsample as needed.
			if ((mData.interlaced_tab[index_tab_2][j]) || (mData.interlaced_tab[index_tab_0][j]))
			{
				dst_U[j]=239;
				dst_V[j]=239;
			}
			else
			{
				dst_U[j]=(unsigned char)((lookup[src_Un[j]]+(unsigned short)src_Unn[j]+2)>>2);
				dst_V[j]=(unsigned char)((lookup[src_Vn[j]]+(unsigned short)src_Vnn[j]+2)>>2);
			}
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		index_tab_0=(index_tab_0+2)%Interlaced_Tab_Size;
		index_tab_1=(index_tab_1+2)%Interlaced_Tab_Size;
		index_tab_2=(index_tab_2+2)%Interlaced_Tab_Size;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

	for(i=h_4; i<h_Y; i+=4)
	{
		for(j=0; j<w_UV; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_U[j]+512]+(unsigned short)src_Upp[j]+4)>>3);
			dst_V[j]=(unsigned char)((lookup[src_V[j]+512]+(unsigned short)src_Vpp[j]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		for(j=0; j<w_UV; j++)
		{
			dst_U[j]=(unsigned char)((lookup[src_Up[j]]+lookup[src_Un[j]+256]+4)>>3);
			dst_V[j]=(unsigned char)((lookup[src_Vp[j]]+lookup[src_Vn[j]+256]+4)>>3);
		}
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_U,w_UV);
		memcpy(dst_V,src_V,w_UV);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		memcpy(dst_U,src_Un,w_UV);
		memcpy(dst_V,src_Vn,w_UV);
		dst_U+=dst_pitch_U;
		dst_V+=dst_pitch_V;

		src_U+=pitch_U_2;
		src_Up+=pitch_U_2;
		src_Upp+=pitch_U_2;
		src_Un+=pitch_U_2;
		src_Unn+=pitch_U_2;
		src_Unnn+=pitch_U_2;

		src_V+=pitch_V_2;
		src_Vp+=pitch_V_2;
		src_Vpp+=pitch_V_2;
		src_Vn+=pitch_V_2;
		src_Vnn+=pitch_V_2;
		src_Vnnn+=pitch_V_2;
	}

}

void JPSDR_AutoYUY2::Run()
{
	Image_Data idata;

	if (g_VFVAPIVersion<12) return;

	idata=mData.image_data;

	if ((idata.src_h0==0) || (idata.src_w0==0)) return;

	if (g_VFVAPIVersion>=12)
	{
		const VDXPixmap& pxdst=*fa->dst.mpPixmap;
		const VDXPixmap& pxsrc=*fa->src.mpPixmap;

		idata.src_plane0=pxsrc.data;
		idata.src_plane1=pxsrc.data2;
		idata.src_plane2=pxsrc.data3;
		idata.dst_plane0=pxdst.data;
		idata.dst_plane1=pxdst.data2;
		idata.dst_plane2=pxdst.data3;
	}
	else
	{
		idata.src_plane0=fa->src.data;
		idata.dst_plane0=fa->dst.data;
		idata.src_plane1=NULL;
		idata.src_plane2=NULL;
		idata.dst_plane1=NULL;
		idata.dst_plane2=NULL;

	}

	switch(idata.dst_video_mode)
	{
		case 2 :
			switch(mData.convert_mode)
			{
				case 0 :
					Convert_Progressive_to_YUY2(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.src_w0,
						idata.src_h0,idata.src_pitch0,idata.src_pitch1,idata.src_pitch2,idata.dst_pitch0);
					break;
				case 1 :
					Convert_Interlaced_to_YUY2(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.src_w0,
						idata.src_h0,idata.src_pitch0,idata.src_pitch1,idata.src_pitch2,idata.dst_pitch0);
					break;
				case 2 :
					Convert_Automatic_to_YUY2(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.src_w0,
						idata.src_h0,idata.src_pitch0,idata.src_pitch1,idata.src_pitch2,idata.dst_pitch0,mData.threshold);
					break;
				case 3 :
					Convert_Test_to_YUY2(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.src_w0,
						idata.src_h0,idata.src_pitch0,idata.src_pitch1,idata.src_pitch2,idata.dst_pitch0,mData.threshold);
					break;
			}
			break;
		case 3 :
			switch(mData.convert_mode)
			{
				case 0 :
					Convert_Progressive_to_UYVY(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.src_w0,
						idata.src_h0,idata.src_pitch0,idata.src_pitch1,idata.src_pitch2,idata.dst_pitch0);
					break;
				case 1 :
					Convert_Interlaced_to_UYVY(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.src_w0,
						idata.src_h0,idata.src_pitch0,idata.src_pitch1,idata.src_pitch2,idata.dst_pitch0);
					break;
				case 2 :
					Convert_Automatic_to_UYVY(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.src_w0,
						idata.src_h0,idata.src_pitch0,idata.src_pitch1,idata.src_pitch2,idata.dst_pitch0,mData.threshold);
					break;
				case 3 :
					Convert_Test_to_UYVY(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.src_w0,
						idata.src_h0,idata.src_pitch0,idata.src_pitch1,idata.src_pitch2,idata.dst_pitch0,mData.threshold);
					break;
			}
			break;
		case 5 :
			switch(mData.convert_mode)
			{
				case 0 :
					Convert_Progressive_to_Planar422(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,
						idata.dst_plane1,idata.dst_plane2,idata.src_w0,idata.src_w1,idata.src_w2,idata.src_h0,idata.src_modulo0,
						idata.src_pitch1,idata.src_pitch2,idata.dst_modulo0,idata.dst_pitch1,idata.dst_pitch2);
					break;
				case 1 :
					Convert_Interlaced_to_Planar422(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,
						idata.dst_plane1,idata.dst_plane2,idata.src_w0,idata.src_w1,idata.src_w2,idata.src_h0,idata.src_modulo0,
						idata.src_pitch1,idata.src_pitch2,idata.dst_modulo0,idata.dst_pitch1,idata.dst_pitch2);
					break;
				case 2 :
					Convert_Automatic_to_Planar422(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.dst_plane1,
						idata.dst_plane2,idata.src_w0,idata.src_w1,idata.src_h0,idata.src_modulo0,idata.src_pitch1,idata.src_pitch2,
						idata.dst_modulo0,idata.dst_pitch1,idata.dst_pitch2,mData.threshold);
					break;
				case 3 :
					Convert_Test_to_Planar422(idata.src_plane0,idata.src_plane1,idata.src_plane2,idata.dst_plane0,idata.dst_plane1,
						idata.dst_plane2,idata.src_w0,idata.src_w1,idata.src_h0,idata.src_modulo0,idata.src_pitch1,idata.src_pitch2,
						idata.dst_modulo0,idata.dst_pitch1,idata.dst_pitch2,mData.threshold);
					break;
			}
			break;
		default : break;
	}

}




void JPSDR_AutoYUY2::End()
{
	signed char i;

	for (i=Interlaced_Tab_Size-1; i>=0; i--)
	{
		myfree(mData.interlaced_tab[i]);
	}
}



void JPSDR_AutoYUY2::ScriptConfig(IVDXScriptInterpreter *isi, const VDXScriptValue *argv, int argc)
{
	mData.output_format=argv[0].asInt();
	mData.convert_mode=argv[1].asInt();
	mData.threshold=argv[2].asInt();
}

bool JPSDR_AutoYUY2::Configure(VDXHWND hwnd)
{
	JPSDR_AutoYUY2Dialog dlg(mData, fa->ifp);
	
	return dlg.Show((HWND)hwnd);
}


void JPSDR_AutoYUY2::GetScriptString(char *buf, int maxlen)
{
 
  
    SafePrintf(buf, maxlen, "Config(%d, %d, %d)",
		mData.output_format,mData.convert_mode,mData.threshold);
}



extern VDXFilterDefinition filterDef_JPSDR_AutoYUY2=
VDXVideoFilterDefinition<JPSDR_AutoYUY2>("JPSDR","Auto YUY2 v1.0.0","Convertion Planar420 to 422 mode. [SSE] Optimised.");

